
/******************************  LICENSE  *******************************

 * (C) Copyright 1996-2017 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.

 *******************************  LICENSE  *******************************/

/*! \\file MagicsGlobalAttributes.h
    \\brief Definition of MagicsGlobal Attributes class.

    This file is automatically generated.
    Do Not Edit!
*/

#include "MagRequest.h" 
#include "MagicsGlobalWrapper.h"
#include "MagicsParameter.h"
#include "Factory.h"
#include "MagTranslator.h"

using namespace magics;


MagicsGlobalWrapper::MagicsGlobalWrapper(): magicsglobal_(new MagicsGlobal())

{

	
	
	
} 
MagicsGlobalWrapper::MagicsGlobalWrapper(MagicsGlobal* magicsglobal): magicsglobal_(magicsglobal)
{
	
	
} 

MagicsGlobalWrapper::~MagicsGlobalWrapper()
{
	
}

void MagicsGlobalWrapper::set(const MagRequest& request)
{
	
	

	if  (request.countValues("MAGICS_SILENT") ) {
		string silent_value = request("MAGICS_SILENT");
		magicsglobal_->silent_ = MagTranslator<string, bool>()(silent_value);
		}
	if  (request.countValues("MAGICS_BACKWARD_COMPATIBILITY") ) {
		string compatibility_value = request("MAGICS_BACKWARD_COMPATIBILITY");
		magicsglobal_->compatibility_ = MagTranslator<string, bool>()(compatibility_value);
		}
	
	
}

void MagicsGlobalWrapper::print(ostream& out)  const
{
	out << "MagicsGlobalWrapper[]";
}


    



