
/******************************  LICENSE  *******************************

 * (C) Copyright 1996-2017 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.

 *******************************  LICENSE  *******************************/

/*! \\file NoPageIDAttributes.h
    \\brief Definition of NoPageID Attributes class.

    This file is automatically generated.
    Do Not Edit!
*/

#include "MagRequest.h" 
#include "NoPageIDWrapper.h"
#include "MagicsParameter.h"
#include "Factory.h"
#include "MagTranslator.h"

using namespace magics;


NoPageIDWrapper::NoPageIDWrapper(): nopageid_(new NoPageID())

{

	
	
	
} 
NoPageIDWrapper::NoPageIDWrapper(NoPageID* nopageid): nopageid_(nopageid)
{
	
	
} 

NoPageIDWrapper::~NoPageIDWrapper()
{
	
}

void NoPageIDWrapper::set(const MagRequest& request)
{
	
	

	
	
}

void NoPageIDWrapper::print(ostream& out)  const
{
	out << "NoPageIDWrapper[]";
}


    



