
/******************************  LICENSE  *******************************

 * (C) Copyright 1996-2017 ECMWF.
 * 
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
 * In applying this licence, ECMWF does not waive the privileges and immunities 
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.

 *******************************  LICENSE  *******************************/

/*! \\file ListColourTechniqueAttributes.h
    \\brief Definition of ListColourTechnique Attributes class.

    This file is automatically generated.
    Do Not Edit!
*/

#include "MagRequest.h" 
#include "ListColourTechniqueWrapper.h"
#include "MagicsParameter.h"
#include "Factory.h"
#include "MagTranslator.h"

using namespace magics;


ListColourTechniqueWrapper::ListColourTechniqueWrapper(): listcolourtechnique_(new ListColourTechnique())

{

	
	
	ColourTechniqueWrapper::object(listcolourtechnique_);
	
	
} 
ListColourTechniqueWrapper::ListColourTechniqueWrapper(ListColourTechnique* listcolourtechnique): listcolourtechnique_(listcolourtechnique)
{
	
	
	ColourTechniqueWrapper::object(listcolourtechnique_);
	
} 

ListColourTechniqueWrapper::~ListColourTechniqueWrapper()
{
	
}

void ListColourTechniqueWrapper::set(const MagRequest& request)
{
	
	
	
	ColourTechniqueWrapper::set(request);
	

	stringarray  values_value;
	for (int i = 0; i < request.countValues("CONTOUR_SHADE_COLOUR_LIST"); i++)
		values_value.push_back((string)request("CONTOUR_SHADE_COLOUR_LIST", i));
	if ( !values_value.empty() ) 
		listcolourtechnique_->values_ = values_value;
	
	
}

void ListColourTechniqueWrapper::print(ostream& out)  const
{
	out << "ListColourTechniqueWrapper[]";
}


    



