/*
 * (C) Copyright 1996-2016 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 * In applying this licence, ECMWF does not waive the privileges and immunities
 * granted to it by virtue of its status as an intergovernmental organisation nor
 * does it submit to any jurisdiction.
 */

/*! \file MagLogoPlottingBase.h
    \brief Definition of the Template class MagLogoPlottingBase.

    Magics Team - ECMWF 2006

    Started: Thu 9-Feb-2006

    Changes:

*/

#ifndef MagLogoPlottingBase_H
#define MagLogoPlottingBase_H

#include "BasicGraphicsObject.h"
#include "Factory.h"
#include "MagTranslator.h"
#include "magics.h"


namespace magics {}  // namespace magics
#endif
